@@ -3,23 +3,60 @@
 #include <rxode2.h>
 #include <float.h>
 
-#define _evid (&_solveData->subjects[_cSub])->evid[(&_solveData->subjects[_cSub])->ix[(&_solveData->subjects[_cSub])->idx]]
-#define amt (isDose(_evid) ?  (&_solveData->subjects[_cSub])->dose[(&_solveData->subjects[_cSub])->ixds] : NA_REAL)
+#define _evid getEvid((&_solveData->subjects[_cSub]), (&_solveData->subjects[_cSub])->ix[(&_solveData->subjects[_cSub])->idx])
+#define amt (isDose(_evid) ?  getDose((&_solveData->subjects[_cSub]),(&_solveData->subjects[_cSub])->ixds) : NA_REAL)
 #define JAC_Rprintf Rprintf
-#define _idx (&_solveData->subjects[_cSub])->idx
-#define JAC0_Rprintf if ( (&_solveData->subjects[_cSub])->jac_counter == 0) Rprintf
+#define _idx (_solveData->subjects[_cSub]).idx
+#define JAC0_Rprintf if ((_solveData->subjects[_cSub]).jac_counter == 0) Rprintf
 #define ODE_Rprintf Rprintf
-#define ODE0_Rprintf if ( (&_solveData->subjects[_cSub])->dadt_counter == 0) Rprintf
+#define ODE0_Rprintf if ((_solveData->subjects[_cSub]).dadt_counter == 0) Rprintf
 #define LHS_Rprintf Rprintf
-#define _safe_log(a) (&_solveData->safeZero ? (((a) <= 0) ? log(DBL_EPSILON) : log(a)) : log(a))
-#define safe_zero(a) (&_solveData->safeZero ? ((a) == 0 ? DBL_EPSILON : ((double)a)) : ((double)a))
-#define _as_zero(a) (&_solveData->safeZero && fabs(a) < sqrt(DBL_EPSILON) ? 0.0 : ((double)a))
-#define _as_dbleps(a) (&_solveData->safeZero && fabs(a) < sqrt(DBL_EPSILON) ? (((double)a) < 0 ? -sqrt(DBL_EPSILON)  : sqrt(DBL_EPSILON)) : ((double) a))
-#define _as_dbleps2(a) (&_solveData->safeZero && fabs(a) < sqrt(DBL_EPSILON) ? sqrt(DBL_EPSILON) : ((double)a))
+static inline double _safe_log_(double a, rx_solve *rx) {
+  if (rx->safeLog) {
+    return (a <= 0) ? log(DBL_EPSILON) : log(a);
+  } else {
+    return log(a);
+  }
+}
+#define _safe_log(a) _safe_log_(a, _solveData)
+static inline double _div0_(double denom, rx_solve *rx) {
+  if (rx->safeZero) {
+    return (denom == 0.0) ? DBL_EPSILON : denom;
+  } else {
+    return denom;
+  }
+}
+#define _div0(a) _div0_(a, _solveData)
 #define factorial(a) exp(lgamma1p(a))
 #define sign_exp(sgn, x)(((sgn) > 0.0) ? exp(x) : (((sgn) < 0.0) ? -exp(x) : 0.0))
-#define Rx_pow(a, b) (&_solveData->useStdPow ? pow(a, b) : R_pow(a, b))
-#define Rx_pow_di(a, b) (&_solveData->useStdPow ? pow(a, b) : R_pow_di(a, b))
+static inline double Rx_pow_(double a, double b, rx_solve *rx) {
+  double a0 = a;
+  if (rx->safePow) {
+    if (b < 0 && a == 0.0) {
+      a0 = DBL_EPSILON;
+    }
+  }
+  if (rx->useStdPow) {
+    return pow(a0, b);
+  } else {
+    return R_pow(a0, b);
+  }
+}
+#define Rx_pow(a, b) Rx_pow_(a, b, _solveData)
+static inline double Rx_pow_di_(double a, double b, rx_solve *rx) {
+  double a0 = a;
+  if (rx->safePow) {
+    if (b < 0 && a == 0.0) {
+      a0 = DBL_EPSILON;
+    }
+  }
+  if (rx->useStdPow) {
+    return pow(a0, b);
+  } else {
+    return R_pow_di(a0, (int)b);
+  }
+}
+#define Rx_pow_di(a, b) Rx_pow_di_(a, b, _solveData)
 #define abs_log1p(x) (((x) + 1.0 > 0.0) ? log1p(x) : (((x) + 1.0 > 0.0) ? log1p(-x) : 0.0))
 #define abs_log(x) ((&_solveData->safeZero && fabs(x) <= sqrt(DBL_EPSILON)) ? log(sqrt(DBL_EPSILON)) : (((x) > 0.0) ? log(x) ? (((x) == 0) ? 0.0 : log(-x))))
 #define _IR (_solveData->subjects[_cSub].InfusionRate)
@@ -55,8 +92,8 @@
 // FIXME: need to use same scheme here
 #define rnormV(ind, x,y) rxnormV(ind,x,y)
 #define rnormV1(ind, id, x) rxnormV(ind, id, x, 1.0)
-  
-#undef rcauchy 
+
+#undef rcauchy
 #define rcauchy(ind, x, y) rxcauchy(ind,x,y)
 #define rxcauchy1(x) rxcauchy(&_solveData->subjects[_cSub],x, 1.0)
 #define ricauchy1(id, x) ricauchy(&_solveData->subjects[_cSub], id, x, 1.0)
@@ -186,12 +223,28 @@
 #define _dose1(x) (_solveData->subjects[_cSub].curDoseS[x])
 #define _tad0() (t-_solveData->subjects[_cSub].tlast)
 #define _tad1(x) (t-_solveData->subjects[_cSub].tlastS[x])
+#define _tad00() (ISNA(_solveData->subjects[_cSub].tlast)? 0 : (t- _solveData->subjects[_cSub].tlast))
+#define _tad01(x) (ISNA(_solveData->subjects[_cSub].tlastS[x]) ? 0 : (t- _solveData->subjects[_cSub].tlastS[x]))
 #define _tafd0()  (t-_solveData->subjects[_cSub].tfirst)
+#define _tafd00()  (ISNA(_solveData->subjects[_cSub].tfirst) ? 0 : (t- _solveData->subjects[_cSub].tfirst))
 #define _tafd1(x) (t-_solveData->subjects[_cSub].tfirstS[x])
+#define _tafd01(x) (ISNA(_solveData->subjects[_cSub].tfirstS[x]) ? 0 : (t- _solveData->subjects[_cSub].tfirstS[x]))
 #define _tlast0() _solveData->subjects[_cSub].tlast
 #define _tlast1(x) _solveData->subjects[_cSub].tlastS[x]
+#define _tlast00() (ISNA(_solveData->subjects[_cSub].tlast) ? 0 : _solveData->subjects[_cSub].tlast)
+#define _tlast01(x) (ISNA(_solveData->subjects[_cSub].tlastS[x]) ? 0 : _solveData->subjects[_cSub].tlastS[x])
 #define _tfirst0()  _solveData->subjects[_cSub].tfirst
+#define _tfirst00()  (ISNA(_solveData->subjects[_cSub].tfirst) ? 0 : _solveData->subjects[_cSub].tfirst)
 #define _tfirst1(x) _solveData->subjects[_cSub].tfirstS[x]
+#define _tfirst01(x) (ISNA(_solveData->subjects[_cSub].tfirstS[x]) ? 0 : _solveData->subjects[_cSub].tfirstS[x])
+#define rxAlagLin(x) x
+#define rxAlag1Lin(x) x
+#define rxFLin(x) x
+#define rxF1Lin(x) x
+#define rxRateLin(x) x
+#define rxRate1Lin(x) x
+#define rxDurLin(x) x
+#define rxDur1Lin(x) x
 #undef rf
 #define rf(ind, x, y) rxf(ind, x, y)
 // int compareFactorVal(int val, const char *valStr, const char *cmpValue)
@@ -208,7 +261,7 @@ typedef double (*rxode2_fn2) (double x, double y);
 typedef double (*rxode2_fn3) (double x, double y, double z);
 typedef double (*rxode2_fn3i) (double x, double y, int i);
 typedef double (*rxode2_fn2i) (double x, int i);
-typedef int (*rxode2_fn0i) ();
+typedef int (*rxode2_fn0i) (void);
 typedef double (*rxode2i_fn) (rx_solving_options_ind* ind, double x);
 typedef int (*rxode2i_ifn) (rx_solving_options_ind* ind, double x);
 typedef double (*rxode2i_fn2) (rx_solving_options_ind* ind, double x, double y);
@@ -232,6 +285,7 @@ typedef SEXP (*_rxGetModelLibType)(const char *s);
 typedef  SEXP (*_rx_asgn) (SEXP objectSEXP);
 typedef int(*_rxIsCurrentC_type)(SEXP);
 typedef double(*_rxSumType)(double *, int, double *, int, int);
+typedef double(*_udf_type)(const char *fun, int, double *);
 
 typedef void(*_simfun)(int id);
 
@@ -263,7 +317,7 @@ typedef void (*_update_par_ptr_p)(double t, unsigned int id, rx_solve *rx, int i
 
 typedef double (*_getParCov_p)(unsigned int id, rx_solve *rx, int parNo, int idx);
 
-typedef rx_solve *(*_getRxSolve_t)();
+typedef rx_solve *(*_getRxSolve_t)(void);
 
 typedef int (*rxode2i_rxbinom) (rx_solving_options_ind* ind, int n, double prob);
 typedef int (*rxode2i2_ribinom) (rx_solving_options_ind* ind, int id, int n, double prob);
@@ -283,4 +337,7 @@ typedef double (*rxode2_llikWeibullFun) (double *in, double x, double shape, dou
 typedef double (*rxode2_llikGammaFun) (double *in, double x, double shape, double rate);
 typedef double (*rxode2_llikCauchyFun) (double *in, double x, double location, double scale);
 
+
+typedef void (*rxode2_assignFuns2_t)(rx_solve, rx_solving_options, t_F, t_LAG, t_RATE, t_DUR,t_calc_mtime, t_ME, t_IndF, t_getTime, t_locateTimeIndex, t_handle_evidL,t_getDur);
+
 #endif // __rxode2_model_shared_H__
