@@ -6,9 +6,8 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include "../inst/include/rxode2.h"
-#include <rxode2parseHandleEvid.h>
-#include <rxode2parseGetTime.h>
-#include "seed.h"
+#include "../inst/include/rxode2parseHandleEvid.h"
+#include "../inst/include/rxode2parseGetTime.h"
 
 #define safe_zero(a) ((a) == 0 ? DBL_EPSILON : (a))
 #define _as_zero(a) (fabs(a) < sqrt(DBL_EPSILON) ? 0.0 : a)
@@ -28,14 +27,14 @@
 
 void handleTlast(double *time, rx_solving_options_ind *ind);
 
+double rxunif(rx_solving_options_ind* ind, double low, double hi);
+
 // From https://cran.r-project.org/web/packages/Rmpfr/vignettes/log1mexp-note.pdf
 double log1mex(double a){
   if (a < M_LN2) return log(-expm1(-a));
   return(log1p(-exp(-a)));
 }
 
-void getWh(int evid, int *wh, int *cmt, int *wh100, int *whI, int *wh0);
-
 extern int _locateTimeIndex(double obs_time,  rx_solving_options_ind *ind){
   // Uses bisection for slightly faster lookup of dose index.
   int i, j, ij;
@@ -74,43 +73,66 @@ extern int _locateTimeIndex(double obs_time,  rx_solving_options_ind *ind){
    - Use getTime(to allow model-based changes to dose timing
    - Use getValue to ignore NA values for time-varying covariates
 */
-static inline double getValue(int idx, double *y, rx_solving_options_ind *ind, rx_solving_options *op){
+static inline double getValue(int idx, double *y, int is_locf,
+                              rx_solving_options_ind *ind, rx_solving_options *op,
+                              int lh){
   int i = idx;
   double ret = y[ind->ix[idx]];
   if (ISNA(ret)) {
-    if (op->f2 == 1.0 && op->f1 == 0.0) {
-      // use nocb
-      // Go forward
-      while (ISNA(ret) && i != ind->n_all_times-1){
-        i++; ret = y[ind->ix[i]];
+    // NA handling
+    int backward = 1;
+    if (is_locf == 1) {
+      backward = 1; // for locf always get the previous value
+    } else if (is_locf == 2) {
+      backward = 0; // for nocb always get the previous value
+    } else if (is_locf == 0 || is_locf == 3) {
+      // linear & midpoint choose based on direction
+      if (lh == -1 || lh == -2) {
+        // get previous value for the left or centered value
+        backward = 1;
+      } else if (lh == 0) {
+        backward = op->instant_backward;
+      } else {
+        // get next value for the right value for approx
+        backward = 0;
       }
-      if (ISNA(ret)){
-        // Still not found go backward
-        i = idx;
-        while (ISNA(ret) && i != 0){
-          i--; ret = y[ind->ix[i]];
-        }
-      }
-    } else {
+    }
+    if (backward) {
       // Go backward.
-      while (ISNA(ret) && i != 0){
+      while (ISNA(ret) && i != 0) {
         i--; ret = y[ind->ix[i]];
       }
-      if (ISNA(ret)){
+      if (ISNA(ret)) {
         // Still not found go forward.
         i = idx;
         while (ISNA(ret) && i != ind->n_all_times-1){
           i++; ret = y[ind->ix[i]];
         }
       }
-
+    } else {
+      // Go forward
+      while (ISNA(ret) && i != ind->n_all_times-1) {
+        i++; ret = y[ind->ix[i]];
+      }
+      if (ISNA(ret)) {
+        // Still not found go backward
+        i = idx;
+        while (ISNA(ret) && i != 0){
+          i--; ret = y[ind->ix[i]];
+        }
+      }
     }
   }
+  if (lh == -2) {
+    ind->idxLow = i;
+  } else if (lh == 2) {
+    ind->idxHi = i;
+  }
   return ret;
 }
 #define T(i) getTime(id->ix[i], id)
-#define V(i) getValue(i, y, id, Meth)
-double rx_approxP(double v, double *y, int n,
+#define V(i, lh) getValue(i, y, is_locf, id, Meth, lh)
+double rx_approxP(double v, double *y, int is_locf, int n,
                   rx_solving_options *Meth, rx_solving_options_ind *id){
   /* Approximate  y(v),  given (x,y)[i], i = 0,..,n-1 */
   int i, j, ij;
@@ -134,16 +156,34 @@ double rx_approxP(double v, double *y, int n,
 
   /* interpolation */
 
-  if(v == T(j)) return V(j);
-  if(v == T(i)) return V(i);
+  if(v == T(j)) return V(j, 1);
+  if(v == T(i)) return V(i, -1);
   /* impossible: if(T(j) == T(i)) return V(i); */
 
-  if(Meth->kind == 1){ /* linear */
-    return V(i) + (V(j) - V(i)) * ((v - T(i))/(T(j) - T(i)));
-  } else { /* 2 : constant */
-    return (Meth->f1 != 0.0 ? V(i) * Meth->f1 : 0.0)
-      + (Meth->f2 != 0.0 ? V(j) * Meth->f2 : 0.0);
+  switch (is_locf) {
+  case 0: // linear
+    {
+      // in the case of linear the time needs to be adjusted based on any na handling rules
+      // when i = -2 or i = 2 then the index of the na value adjustment is saved.
+      double vi = V(i, -2);
+      double vj = V(j, 2);
+      // These saved values are then used for the adjusted times
+      double ti = T(id->idxLow);
+      double tj = T(id->idxHi);
+      return vi + (vj - vi) * ((v - ti)/(tj - ti));
+    }
+    break;
+  case 1: // locf
+    return V(i, -1);
+    break;
+  case 2: // nocb
+    return V(j, 1);
+    break;
+  case 3: // midpoint
+    return 0.5*(V(i, -1) + V(j, 1));
+    break;
   }
+  return NA_REAL; // nocov
 }/* approx1() */
 
 #undef T
@@ -214,7 +254,11 @@ void _update_par_ptr(double t, unsigned int id, rx_solve *rx, int idxIn) {
         idx = j;
       } else if (isSameTimeOp(v, getTime(ind->ix[i], ind))) {
         idx = i;
-      } else if (op->is_locf == 2) {
+      } else if (op->instant_backward == 0) {
+        // use instant_backward to change the idx too; it does not
+        // change based on covariate
+        // backward=0=locf
+        // backward=1=nocb
         // nocb
         idx = j;
       }  else {
@@ -238,10 +282,12 @@ void _update_par_ptr(double t, unsigned int id, rx_solve *rx, int idxIn) {
     if (op->do_par_cov) {
       for (k = ncov; k--;) {
         if (op->par_cov[k]) {
+          int is_locf = op->par_cov_interp[k];
+          if (is_locf == -1) is_locf = op->is_locf;
           if (rx->sample && rx->par_sample[op->par_cov[k]-1] == 1) {
             // Get or sample id from overall ids
             if (ind->cov_sample[k] == 0) {
-              ind->cov_sample[k] = round(rxodeUnif(ind, 0.0, (double)(rx->nsub*rx->nsim)))+1;
+              ind->cov_sample[k] = round(rxunif(ind, 0.0, (double)(rx->nsub*rx->nsim)))+1;
             }
             indSample = &(rx->subjects[ind->cov_sample[k]-1]);
             idxSample = -1;
@@ -250,11 +296,14 @@ void _update_par_ptr(double t, unsigned int id, rx_solve *rx, int idxIn) {
             idxSample = idx;
           }
           double *y = indSample->cov_ptr + indSample->n_all_times*k;
-          ind->par_ptr[op->par_cov[k]-1] = getValue(idxSample, y, indSample, op);
+          ind->par_ptr[op->par_cov[k]-1] = getValue(idxSample, y, is_locf,
+                                                    indSample, op, 0);
           if (idx == 0){
             ind->cacheME=0;
-          } else if (!isSameTimeOp(getValue(idxSample, y, indSample, op),
-                                   getValue(idxSample-1, y, indSample, op))) {
+          } else if (!isSameTimeOp(getValue(idxSample, y, is_locf,
+                                            indSample, op, 0),
+                                   getValue(idxSample-1, y, is_locf,
+                                            indSample, op, 0))) {
             ind->cacheME=0;
           }
         }
@@ -266,11 +315,13 @@ void _update_par_ptr(double t, unsigned int id, rx_solve *rx, int idxIn) {
     int ncov = op->ncov;
     if (op->do_par_cov) {
       for (k = ncov; k--;){
-        if (op->par_cov[k]){
+        if (op->par_cov[k]) {
+          int is_locf = op->par_cov_interp[k];
+          if (is_locf == -1) is_locf = op->is_locf;
           if (rx->sample && rx->par_sample[op->par_cov[k]-1] == 1) {
             // Get or sample id from overall ids
             if (ind->cov_sample[k] == 0) {
-              ind->cov_sample[k] = (int)rxodeUnif(ind, (double)1, (double)(rx->nsub*rx->nsim+1));
+              ind->cov_sample[k] = (int)(rxunif(ind, (double)1, (double)(rx->nsub*rx->nsim+1)));
             }
             indSample = &(rx->subjects[ind->cov_sample[k]-1]);
             idxSample = -1;
@@ -287,16 +338,22 @@ void _update_par_ptr(double t, unsigned int id, rx_solve *rx, int idxIn) {
             ind->cacheME=0;
           } else if (idxSample > 0 && idxSample < indSample->n_all_times &&
                      isSameTimeOp(t, getTime(ind->ix[idxSample], indSample))) {
-            par_ptr[op->par_cov[k]-1] = getValue(idxSample, y, indSample, op);
-            if (!isSameTimeOp(getValue(idxSample, y, indSample, op),
-                              getValue(idxSample-1, y, indSample, op))) {
+            par_ptr[op->par_cov[k]-1] = getValue(idxSample, y, is_locf,
+                                                 indSample, op, 0);
+            if (!isSameTimeOp(getValue(idxSample, y, is_locf,
+                                       indSample, op, 0),
+                              getValue(idxSample-1, y, is_locf,
+                                       indSample, op, 0))) {
               ind->cacheME=0;
             }
           } else {
             // Use the same methodology as approxfun.
-            indSample->ylow = getValue(0, y, indSample, op);/* cov_ptr[ind->n_all_times*k]; */
-            indSample->yhigh = getValue(indSample->n_all_times-1, y, indSample, op);/* cov_ptr[ind->n_all_times*k+ind->n_all_times-1]; */
-            par_ptr[op->par_cov[k]-1] = rx_approxP(t, y, indSample->n_all_times, op, indSample);
+            indSample->ylow = getValue(0, y, is_locf,
+                                       indSample, op, -1);/* cov_ptr[ind->n_all_times*k]; */
+            indSample->yhigh = getValue(indSample->n_all_times-1, y, is_locf,
+                                        indSample, op, 1);/* cov_ptr[ind->n_all_times*k+ind->n_all_times-1]; */
+            par_ptr[op->par_cov[k]-1] = rx_approxP(t, y, is_locf,
+                                                   indSample->n_all_times, op, indSample);
             // Don't need to reset ME because solver doesn't use the
             // times in-between.
           }
