@@ -1,10 +1,57 @@
 #define USE_FC_LEN_T
 #define STRICT_R_HEADERS
 #include "codegen.h"
+
+SEXP _rxode2parse_rxFunctionName;
+SEXP _rxode2parse_functionName;
+SEXP _rxode2parse_functionType;
+SEXP _rxode2parse_functionPackageName;
+SEXP _rxode2parse_functionPackageFunction;
+SEXP _rxode2parse_functionArgMin;
+SEXP _rxode2parse_functionArgMax;
+SEXP _rxode2parse_functionThreadSafe;
+
+SEXP _rxode2parse_funName;
+SEXP _rxode2parse_funNameInt;
+
+SEXP _rxode2parse_packages;
+
+#undef df
+
+SEXP getRxode2ParseDfBuiltin(void);
+
+int _rxode2parse_protected = 0;
+void _rxode2parse_assignTranslationBuiltin(void) {
+  SEXP df = getRxode2ParseDfBuiltin();
+  _rxode2parse_funName = PROTECT(VECTOR_ELT(df, 0)); _rxode2parse_protected++;
+  _rxode2parse_funNameInt = PROTECT(VECTOR_ELT(df, 1)); _rxode2parse_protected++;
+}
+
+void _rxode2parse_assignTranslation(SEXP df) {
+  _rxode2parse_unprotect();
+  _rxode2parse_rxFunctionName = PROTECT(VECTOR_ELT(df, 0)); _rxode2parse_protected++;
+  _rxode2parse_functionName = PROTECT(VECTOR_ELT(df, 1)); _rxode2parse_protected++;
+  _rxode2parse_functionType = PROTECT(VECTOR_ELT(df, 2)); _rxode2parse_protected++;
+  _rxode2parse_functionPackageName = PROTECT(VECTOR_ELT(df, 3)); _rxode2parse_protected++;
+  _rxode2parse_functionPackageFunction = PROTECT(VECTOR_ELT(df, 4)); _rxode2parse_protected++;
+  _rxode2parse_functionArgMin = PROTECT(VECTOR_ELT(df, 5)); _rxode2parse_protected++;
+  _rxode2parse_functionArgMax = PROTECT(VECTOR_ELT(df, 6)); _rxode2parse_protected++;
+  _rxode2parse_functionThreadSafe = PROTECT(VECTOR_ELT(df, 7));_rxode2parse_protected++;
+  _rxode2parse_assignTranslationBuiltin();
+}
+
+void _rxode2parse_unprotect(void) {
+  if (_rxode2parse_protected) UNPROTECT(_rxode2parse_protected);
+  _rxode2parse_protected = 0;
+}
+
+
 #include "codegen2.h"
 
 SEXP _rxode2_rxQs(SEXP);
 SEXP _rxode2_rxQr(SEXP);
+SEXP getRxode2ParseDf(void);
+SEXP getRxode2ParseGetPointerAssignment(void);
 
 static FILE *fpIO;
 
@@ -76,17 +123,25 @@ void print_aux_info(char *model, const char *prefix, const char *libname, const
   sFree(&bufw);
 }
 
+extern SEXP getRxode2ParseDf(void);
 
 void codegen(char *model, int show_ode, const char *prefix, const char *libname, const char *pMd5, const char *timeId, const char *libname2) {
-  if (show_ode == 4) {
+  _rxode2parse_assignTranslation(getRxode2ParseDf());
+  _rxode2parse_packages = getRxode2ParseGetPointerAssignment();
+  if (show_ode == ode_printaux) {
     print_aux_info(model, prefix, libname, pMd5, timeId, libname2);
   } else {
     int i, j;
     char *buf;
-    if (show_ode == 1){
+    if (show_ode == ode_dydt){
       const char *extra = "";
       if (strncmp("rx_", libname, 3) != 0) extra = libname;
       writeHeader(md5, extra);
+      for (int i = Rf_length(_rxode2parse_functionName); i--;) {
+        sAppend(&sbOut, "#define %s _rx%s%s%ld\n",
+                R_CHAR(STRING_ELT(_rxode2parse_functionName, i)),
+                extra, md5, __timeId++);
+      }
       sAppendN(&sbOut,"#include <rxode2_model_shared.h>\n",33);
       int mx = maxSumProdN;
       if (SumProdLD > mx) mx = SumProdLD;
@@ -97,7 +152,7 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         // Hence CMT = 4 could translate in data to 44 with sensi=10
         // Then cmt=44 translates back to cmt-10 or 4.
         // This makes the sensitivity equations insensitive to CMT changes that occur in FOCEi
-        sAppend(&sbOut,"#define _CMT ((abs(CMT)<=%d) ? CMT : ((CMT<0) ? CMT+%d: CMT-%d))\n",
+        sAppend(&sbOut,"#define _CMT ((fabs(CMT)<=%d) ? CMT : ((CMT<0) ? CMT+%d: CMT-%d))\n",
                 baseSize, tb.sensi, tb.sensi);
       } else {
         sAppendN(&sbOut,"#define _CMT CMT\n", 17);
@@ -109,22 +164,40 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
       // Add sync PP define
       prnt_vars(print_simeps, 1, "#define _SYNC_simeps_ for (int _svari=_solveData->neps; _svari--;){", "}\n", 15);
       prnt_vars(print_simeta, 1, "#define _SYNC_simeta_ for (int _ovari=_solveData->neta; _ovari--;){", "}\n", 16);
+      writeBody0();
       sAppendN(&sbOut,"#include \"extraC.h\"\n", 20);
-      writeBody();
+      writeBody1();
+      for (int i = Rf_length(_rxode2parse_functionName); i--;) {
+        sAppend(&sbOut,"  %s = (%s) R_GetCCallable(\"%s\", \"%s\");\n",
+                R_CHAR(STRING_ELT(_rxode2parse_functionName, i)),
+                R_CHAR(STRING_ELT(_rxode2parse_functionType, i)),
+                R_CHAR(STRING_ELT(_rxode2parse_functionPackageName, i)),
+                R_CHAR(STRING_ELT(_rxode2parse_functionPackageFunction, i)));
+      }
+      writeBody2();
+      for (int i = Rf_length(_rxode2parse_packages); i--;) {
+        const char* cur = R_CHAR(STRING_ELT(_rxode2parse_packages, i));
+        sAppend(&sbOut,"    static rxode2_assignFuns2_t %s_assignFuns2 = NULL;\n", cur);
+        sAppend(&sbOut,"    if (%s_assignFuns2 == NULL) %s_assignFuns2 = (rxode2_assignFuns2_t)(R_GetCCallable(\"%s\", \"_%s_assignFuns2\"));\n",
+                cur, cur, cur, cur);
+        sAppend(&sbOut,"    %s_assignFuns2(rx, op, f, lag, rate, dur, mtime, me, indf, gettime, timeindex, handleEvid, getdur);\n",
+                cur);
+      }
+      writeBody3();
       sAppend(&sbOut, "extern void  %sode_solver_solvedata (rx_solve *solve){\n  _solveData = solve;\n}\n",prefix);
-      sAppend(&sbOut, "extern rx_solve *%sode_solver_get_solvedata(){\n  return _solveData;\n}\n", prefix);
-      sAppend(&sbOut, "SEXP %smodel_vars();\n", prefix);
+      sAppend(&sbOut, "extern rx_solve *%sode_solver_get_solvedata(void){\n  return _solveData;\n}\n", prefix);
+      sAppend(&sbOut, "SEXP %smodel_vars(void);\n", prefix);
       sAppendN(&sbOut,"\n", 1);
       sAppendN(&sbOut, "\n// prj-specific differential eqns\nvoid ", 40);
       sAppend(&sbOut, "%sdydt(int *_neq, double __t, double *__zzStateVar__, double *__DDtStateVar__)\n{\n  int _itwhile = 0;\n  (void)_itwhile;\n  int _cSub = _neq[1];\n  double t = __t + _solveData->subjects[_neq[1]].curShift;\n  (void)t;\n  (&_solveData->subjects[_cSub])->_rxFlag=1;\n  ", prefix);
-    } else if (show_ode == 2){
+    } else if (show_ode == ode_jac){
       sAppend(&sbOut, "// Jacobian derived vars\nvoid %scalc_jac(int *_neq, double __t, double *__zzStateVar__, double *__PDStateVar__, unsigned int __NROWPD__) {\n  int _itwhile = 0;\n  (void)_itwhile;\n    int _cSub=_neq[1];\n  double t = __t + _solveData->subjects[_neq[1]].curShift;\n  (void)t;\n  (&_solveData->subjects[_cSub])->_rxFlag=2;\n  ", prefix);
-    } else if (show_ode == 3){
+    } else if (show_ode == ode_ini){
       sAppend(&sbOut,  "// Functional based initial conditions.\nvoid %sinis(int _cSub, double *__zzStateVar__){\n  int _itwhile = 0;\n  (void)_itwhile;\n  \n  (&_solveData->subjects[_cSub])->_rxFlag=3;\n  ", prefix);
       if (foundF0){
         sAppendN(&sbOut, "  double t=0;\n", 14);
       }
-    } else if (show_ode == 5){
+    } else if (show_ode == ode_fbio){
       if (foundF){
         int nnn = tb.de.n;
         if (tb.linCmt){
@@ -143,7 +216,7 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         sAppend(&sbOut,  "// Functional based bioavailability\ndouble %sF(int _cSub,  int _cmt, double _amt, double __t, double *__zzStateVar__){\n return _amt;\n  ",
                 prefix);
       }
-    } else if (show_ode == 6){
+    } else if (show_ode == ode_lag){
       if (foundLag){
         int nnn = tb.de.n;
         if (tb.linCmt){
@@ -162,7 +235,7 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         sAppend(&sbOut,  "// Functional based absorption lag\ndouble %sLag(int _cSub,  int _cmt, double __t, double *__zzStateVar__){\n return __t;\n",
                 prefix);
       }
-    } else if (show_ode == 7){
+    } else if (show_ode == ode_rate){
       if (foundRate){
         int nnn = tb.de.n;
         if (tb.linCmt){
@@ -181,7 +254,7 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         sAppend(&sbOut,  "// Modeled zero-order rate\ndouble %sRate(int _cSub,  int _cmt, double _amt, double __t, double *__zzStateVar__){\n return 0.0;\n",
                 prefix);
       }
-    } else if (show_ode == 8){
+    } else if (show_ode == ode_dur){
       if (foundDur){
         int nnn = tb.de.n;
         if (tb.linCmt){
@@ -200,7 +273,7 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         sAppend(&sbOut,  "// Modeled zero-order duration\ndouble %sDur(int _cSub,  int _cmt, double _amt, double __t){\n return 0.0;\n",
                 prefix);
       }
-    } else if (show_ode == 9){
+    } else if (show_ode == ode_mtime){
       if (nmtime){
         sAppend(&sbOut,  "// Model Times\nvoid %smtime(int _cSub, double *_mtime){\n  int _itwhile = 0;\n  (void)_itwhile;\n  double t = 0;\n  (&_solveData->subjects[_cSub])->_rxFlag=8;\n  ",
                 prefix);
@@ -208,27 +281,28 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         sAppend(&sbOut,  "// Model Times\nvoid %smtime(int _cSub, double *_mtime){\n",
                 prefix);
       }
-    } else if (show_ode == 10){
+    } else if (show_ode == ode_mexp){
       sAppend(&sbOut, "// Matrix Exponential (%d)\nvoid %sME(int _cSub, double _t, double __t, double *_mat, const double *__zzStateVar__){\n  int _itwhile = 0;\n  (void)_itwhile;\n  double t = __t + _solveData->subjects[_cSub].curShift;\n  (void)t;\n  (&_solveData->subjects[_cSub])->_rxFlag=9;\n  ",
               tb.matn, prefix);
-    } else if (show_ode == 11){
+    } else if (show_ode == ode_indLinVec) {
       sAppend(&sbOut, "// Inductive linearization Matf\nvoid %sIndF(int _cSub, double _t, double __t, double *_matf){\n int _itwhile = 0;\n  (void)_itwhile;\n  double t = __t + _solveData->subjects[_cSub].curShift;\n  (void)t;\n  (&_solveData->subjects[_cSub])->_rxFlag=10;\n  ", prefix);
     } else {
       sAppend(&sbOut,  "// prj-specific derived vars\nvoid %scalc_lhs(int _cSub, double __t, double *__zzStateVar__, double *_lhs) {\n    int _itwhile = 0;\n  (void)_itwhile;\n  double t = __t + _solveData->subjects[_cSub].curShift;\n  (void)t;\n  (&_solveData->subjects[_cSub])->_rxFlag=11;\n  ", prefix);
     }
-    if ((show_ode == 2 && found_jac == 1 && good_jac == 1) ||
-        (show_ode != 2 && show_ode != 3 && show_ode != 5  && show_ode != 8 &&
-         show_ode != 7 && show_ode != 6 &&
-         show_ode !=0 && show_ode != 9 && show_ode != 10 && show_ode != 11) ||
-        (show_ode == 8 && foundDur) ||
-        (show_ode == 7 && foundRate) ||
-        (show_ode == 6 && foundLag) ||
-        (show_ode == 5 && foundF) ||
-        (show_ode == 3 && foundF0) ||
-        (show_ode == 0 && tb.li) ||
-        (show_ode == 9 && nmtime) ||
-        (show_ode == 10 && tb.matn) ||
-        (show_ode == 11 && tb.matnf)){
+    if ((show_ode == ode_jac && found_jac == 1 && good_jac == 1) ||
+        (show_ode != ode_jac && show_ode != ode_ini && show_ode != ode_fbio &&
+         show_ode != ode_dur && show_ode != ode_rate && show_ode != ode_lag &&
+         show_ode != ode_lhs && show_ode != ode_mtime && show_ode != ode_mexp &&
+         show_ode != ode_indLinVec) ||
+        (show_ode == ode_dur && foundDur) ||
+        (show_ode == ode_rate && foundRate) ||
+        (show_ode == ode_lag && foundLag) ||
+        (show_ode == ode_fbio && foundF) ||
+        (show_ode == ode_ini && foundF0) ||
+        (show_ode == ode_lhs && tb.li) ||
+        (show_ode == ode_mtime && nmtime) ||
+        (show_ode == ode_mexp && tb.matn) ||
+        (show_ode == ode_indLinVec && tb.matnf)){
       prnt_vars(print_double, 0, "", "\n",show_ode);     /* declare all used vars */
       if (maxSumProdN > 0 || SumProdLD > 0){
         int mx = maxSumProdN;
@@ -238,6 +312,9 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         sAppend(&sbOut,  "  for (int ddd=%d; ddd--;){_p[ddd]=_input[ddd]=_pld[ddd]=0.0;}", mx);
 
       }
+      if (maxUdf > 0) {
+        sAppend(&sbOut,  "  double __udf[%d];\n", maxUdf);
+      }
       else prnt_vars(print_void, 0, "  (void)t;\n", "\n",show_ode);     /* declare all used vars */
       if (maxSumProdN){
         sAppendN(&sbOut,  "  (void)_p;\n  (void)_input;\n", 28);
@@ -246,30 +323,32 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         }
       }
       prnt_vars(print_lastLhsValue, 0,"","\n", 12);
-      if (show_ode == 3){
+      if (show_ode == ode_ini){
         sAppendN(&sbOut, "  _update_par_ptr(0.0, _cSub, _solveData, _idx);\n", 49);
-      } else if (show_ode == 6 || show_ode == 7 || show_ode == 8 || show_ode == 9){
+      } else if (show_ode == ode_lag || show_ode == ode_rate || show_ode == ode_dur ||
+                 show_ode == ode_mtime){
         // functional lag, rate, duration, mtime
         sAppendN(&sbOut, "  _update_par_ptr(NA_REAL, _cSub, _solveData, _idx);\n", 53);
-      } else if (show_ode == 11 || show_ode == 10){
+      } else if (show_ode == ode_indLinVec || show_ode == ode_mexp){
         sAppendN(&sbOut, "  _update_par_ptr(_t, _cSub, _solveData, _idx);\n", 48);
       } else {
         sAppendN(&sbOut, "  _update_par_ptr(__t, _cSub, _solveData, _idx);\n", 49);
       }
       prnt_vars(print_populateParameters, 1, "", "\n",show_ode);                   /* pass system pars */
-      if (show_ode != 9 && show_ode != 11){
+      if (show_ode != ode_mtime && show_ode != ode_indLinVec){
         for (i=0; i<tb.de.n; i++) {                   /* name state vars */
           buf = tb.ss.line[tb.di[i]];
           if(tb.idu[i] != 0){
-            if (show_ode == 6 || show_ode == 8 || show_ode == 7){
+            if (show_ode == ode_lag || show_ode == ode_dur || show_ode == ode_rate){
               sAppendN(&sbOut, "  ", 2);
+              int assignOne = 0;
               doDot(&sbOut, buf);
               sAppend(&sbOut, " = NA_REAL;\n", i, i);
             } else {
               // stateExtra
               sAppendN(&sbOut, "  ", 2);
               doDot(&sbOut, buf);
-              sAppend(&sbOut, " = __zzStateVar__[%d]*((double)(_ON[%d]));\n", i, i);
+              sAppend(&sbOut, " = __zzStateVar__[__DDT%d__]*((double)(_ON[__DDT%d__]));\n", i, i);
             }
           } else {
             break;
@@ -278,90 +357,96 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         sAppendN(&sbOut, "\n", 1);
       }
     }
-    if ((foundDur && show_ode == 8) ||
-        (foundRate && show_ode == 7) ||
-        (foundLag && show_ode == 6) ||
-        (foundF && show_ode == 5) ||
-        (foundF0 && show_ode == 3) ||
-        (show_ode == 0 && tb.li) ||
-        (show_ode == 9 && nmtime) ||
-        (show_ode == 2 && found_jac == 1 && good_jac == 1) ||
-        (show_ode != 9 && show_ode != 0 && show_ode != 2 && show_ode != 3 && show_ode != 5 && show_ode != 6  && show_ode != 7 && show_ode != 8)){
+    if ((foundDur && show_ode == ode_dur) ||
+        (foundRate && show_ode == ode_rate) ||
+        (foundLag && show_ode == ode_lag) ||
+        (foundF && show_ode == ode_fbio) ||
+        (foundF0 && show_ode == ode_ini) ||
+        (show_ode == ode_lhs && tb.li) ||
+        (show_ode == ode_mtime && nmtime) ||
+        (show_ode == ode_jac && found_jac == 1 && good_jac == 1) ||
+        (show_ode != ode_mtime && show_ode != ode_lhs &&
+         show_ode != ode_jac && show_ode != ode_ini &&
+         show_ode != ode_fbio && show_ode != ode_lag  &&
+         show_ode != ode_rate && show_ode != ode_dur)){
       for (i = 0; i < sbPm.n; i++){
         switch(sbPm.lType[i]){
         case TLIN:
-          if (show_ode != 10 && show_ode != 11 &&
-              show_ode != 5 && show_ode != 6 &&
-              show_ode != 7 && show_ode !=8){
-            sAppend(&sbOut,"  %s",show_ode == 1 ? sbPm.line[i] : sbPmDt.line[i]);
+          if (show_ode != ode_mexp && show_ode != ode_indLinVec &&
+              show_ode != ode_fbio && show_ode != ode_lag &&
+              show_ode != ode_rate && show_ode != ode_dur){
+            sAppend(&sbOut,"  %s",show_ode == ode_dydt ? sbPm.line[i] : sbPmDt.line[i]);
           }
           break;
         case TMTIME:
         case TASSIGN:
-          if (show_ode != 10 && show_ode != 11){
-            sAppend(&sbOut,"  %s",show_ode == 1 ? sbPm.line[i] : sbPmDt.line[i]);
+          if (show_ode != ode_mexp && show_ode != ode_indLinVec){
+            sAppend(&sbOut,"  %s",show_ode == ode_dydt ? sbPm.line[i] : sbPmDt.line[i]);
           }
           break;
         case TINI:
           // See if this is an ini or a reclaimed expression.
-          if (show_ode != 10 && show_ode != 11){
+          if (show_ode != ode_mexp && show_ode != ode_indLinVec){
             if (sbPm.lProp[i] >= 0 ){
               tb.ix = sbPm.lProp[i];
-              if (tb.lh[tb.ix] == isLHS || tb.lh[tb.ix] == isLHSparam){
-                sAppend(&sbOut,"  %s",show_ode == 1 ? sbPm.line[i] : sbPmDt.line[i]);
+              if (tb.lh[tb.ix] == isLHS || tb.lh[tb.ix] == isLHSstr ||
+                  tb.lh[tb.ix] == isLHSparam){
+                sAppend(&sbOut,"  %s",show_ode == ode_dydt ? sbPm.line[i] : sbPmDt.line[i]);
               }
             }
           }
           break;
         case TF0:
           // functional ini
-          if (show_ode == 3) sAppend(&sbOut,"  %s",sbPmDt.line[i]);
+          if (show_ode == ode_ini) sAppend(&sbOut,"  %s",sbPmDt.line[i]);
           break;
         case FBIO:
-          if (show_ode == 5) sAppend(&sbOut,"  %s", sbPmDt.line[i]);
+          if (show_ode == ode_fbio) sAppend(&sbOut,"  %s", sbPmDt.line[i]);
           break;
         case ALAG:
-          if (show_ode == 6) sAppend(&sbOut, "  %s", sbPmDt.line[i]);
+          if (show_ode == ode_lag) sAppend(&sbOut, "  %s", sbPmDt.line[i]);
           break;
         case RATE:
-          if (show_ode == 7) sAppend(&sbOut, "  %s", sbPmDt.line[i]);
+          if (show_ode == ode_rate) sAppend(&sbOut, "  %s", sbPmDt.line[i]);
           break;
         case DUR:
-          if (show_ode == 8) sAppend(&sbOut,"  %s", sbPmDt.line[i]);
+          if (show_ode == ode_dur) sAppend(&sbOut,"  %s", sbPmDt.line[i]);
           break;
         case TJAC:
-          if (show_ode == 0) sAppend(&sbOut, "  %s", sbPmDt.line[i]);
-          else if (show_ode == 2)  sAppend(&sbOut, "  %s", sbPm.line[i]);
+          if (show_ode == ode_lhs) sAppend(&sbOut, "  %s", sbPmDt.line[i]);
+          else if (show_ode == ode_jac)  sAppend(&sbOut, "  %s", sbPm.line[i]);
           break;
         case TDDT:
           // d/dt()
-          if (show_ode != 3 && show_ode != 5 && show_ode != 6 &&
-              show_ode != 7 && show_ode != 8 && show_ode != 9 &&
-              show_ode !=10 && show_ode != 11){
-            sAppend(&sbOut, "  %s", show_ode == 1 ? sbPm.line[i] : sbPmDt.line[i]);
+          if (show_ode != ode_ini && show_ode != ode_fbio && show_ode != ode_lag &&
+              show_ode != ode_rate && show_ode != ode_dur && show_ode != ode_mtime &&
+              show_ode !=10 && show_ode != ode_indLinVec){
+            sAppend(&sbOut, "  %s", show_ode == ode_dydt ? sbPm.line[i] : sbPmDt.line[i]);
           }
           break;
         case PPRN:
           // Rprintf
-          if ((fullPrint && show_ode != 10 && show_ode != 11) || (!fullPrint && show_ode == 1)) {
-            sAppend(&sbOut, "  %s", show_ode == 1 ? sbPm.line[i] : sbPmDt.line[i]);
+          if ((fullPrint && show_ode != ode_mexp && show_ode != ode_indLinVec) || (!fullPrint && show_ode == 1)) {
+            sAppend(&sbOut, "  %s", show_ode == ode_dydt ? sbPm.line[i] : sbPmDt.line[i]);
           }
           break;
         case TLOGIC:
-          if (show_ode != 10 && show_ode != 11){
-            sAppend(&sbOut,"  %s",show_ode == 1 ? sbPm.line[i] : sbPmDt.line[i]);
+          if (show_ode != ode_mexp && show_ode != ode_indLinVec){
+            sAppend(&sbOut,"  %s",show_ode == ode_dydt ? sbPm.line[i] : sbPmDt.line[i]);
           }
           break;
         case TMAT0:
-          if (show_ode == 10){
+          if (show_ode == ode_mexp){
             sAppend(&sbOut,"  %s", sbPm.line[i]);
           }
           break;
         case TMATF:
-          if (show_ode == 11){
+          if (show_ode == ode_indLinVec){
             sAppend(&sbOut,"  %s", sbPm.line[i]);
           }
           break;
+        case TNONE: // no code is output (just props)
+          break;
         default:
           RSprintf("line Number: %d\n", i);
           RSprintf("type: %d\n", sbPm.lType[i]);
@@ -372,47 +457,49 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
       }
       // End statements
       switch (show_ode){
-      case 8:
+      case ode_dur:
         // RATE
         sAppendN(&sbOut, "\n  return _dur[_cmt];\n", 22);
         break;
-      case 7:
+      case ode_rate:
         // DUR
         sAppendN(&sbOut, "\n  return _rate[_cmt];\n", 23);
         break;
-      case 6:
+      case ode_lag:
         // Alag
         sAppendN(&sbOut, "\n  return t + _alag[_cmt] - _solveData->subjects[_cSub].curShift;\n", 66);
         break;
-      case 5:
+      case ode_fbio:
         sAppendN(&sbOut, "\n  return _f[_cmt]*_amt;\n", 25);
         break;
       }
     }
-    if (show_ode == 1){
+    if (show_ode == ode_dydt){
       sAppendN(&sbOut,  "  (&_solveData->subjects[_cSub])->dadt_counter[0]++;\n}\n\n", 56);
-    } else if (show_ode == 2){
+    } else if (show_ode == ode_jac){
       //sAppendN(&sbOut, "  free(__ld_DDtStateVar__);\n");
       sAppendN(&sbOut,  "  (&_solveData->subjects[_cSub])->jac_counter[0]++;\n", 52);
       sAppendN(&sbOut,  "}\n", 2);
-    } else if (show_ode == 3){
+    } else if (show_ode == ode_ini){
       if (foundF0){
         for (i = 0; i < tb.de.n; i++) {
           if (tb.idu[i]) {
             buf=tb.ss.line[tb.di[i]];
-            sAppend(&sbOut, "  __zzStateVar__[%d]=((double)(_ON[%d]))*(",i,i);
+            sAppend(&sbOut, "  __zzStateVar__[__DDT%d__]=((double)(_ON[__DDT%d__]))*(",i,i);
             doDot(&sbOut, buf);
             sAppendN(&sbOut,  ");\n", 3);
           }
         }
       }
       sAppendN(&sbOut,  "}\n", 2);
-    } else if (show_ode == 5 || show_ode == 6 || show_ode == 7 || show_ode == 8){
+    } else if (show_ode == ode_fbio || show_ode == ode_lag || show_ode == ode_rate ||
+               show_ode == ode_dur){
       sAppendN(&sbOut,  "}\n", 2);
-    } else if (show_ode == 0 && tb.li){
+    } else if (show_ode == ode_lhs && tb.li){
       sAppendN(&sbOut,  "\n", 1);
       for (i=0, j=0; i<NV; i++) {
-        if (tb.lh[i] != isLHS && tb.lh[i] != isLhsStateExtra && tb.lh[i] != isLHSparam) continue;
+        if (tb.lh[i] != isLHS && tb.lh[i] != isLhsStateExtra &&
+            tb.lh[i] != isLHSparam && tb.lh[i] != isLHSstr) continue;
         buf = tb.ss.line[i];
         sAppend(&sbOut,  "  _lhs[%d]=", j);
         doDot(&sbOut, buf);
@@ -420,7 +507,7 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
         j++;
       }
       sAppendN(&sbOut,  "}\n", 2);
-    } else if (show_ode == 9 && nmtime){
+    } else if (show_ode == ode_mtime && nmtime){
       sAppendN(&sbOut,  "\n", 1);
       for (i=0, j=0; i<NV; i++) {
         if (tb.mtime[i] != 1) continue;
@@ -435,6 +522,7 @@ void codegen(char *model, int show_ode, const char *prefix, const char *libname,
       sAppendN(&sbOut,  "}\n", 2);
     }
   }
+  _rxode2parse_unprotect();
 }
 
 void writeSb(sbuf *sbb, FILE *fp){
@@ -447,6 +535,7 @@ void writeSb(sbuf *sbb, FILE *fp){
     register unsigned written = fwrite(sbb->s + totalWritten, 1, toWrite, fp);
     if( toWrite != written){
       fclose(fp);
+      _rxode2parse_unprotect();
       err_trans("IO error writing parsed C file");
     } else{
       totalWritten += written; // add the written bytes
@@ -454,23 +543,32 @@ void writeSb(sbuf *sbb, FILE *fp){
   }
   if (totalWritten != sbb->o) {
     fclose(fp);
+    _rxode2parse_unprotect();
     err_trans("IO error writing parsed C file");
   }
 }
 
+extern SEXP _goodFuns;
+
 SEXP _rxode2_codegen(SEXP c_file, SEXP prefix, SEXP libname,
-                     SEXP pMd5, SEXP timeId, SEXP mvLast){
+                          SEXP pMd5, SEXP timeId, SEXP mvLast,
+                          SEXP goodFuns) {
+  _goodFuns = PROTECT(goodFuns); _rxode2parse_protected++;
   if (!sbPm.o || !sbNrm.o){
+    _rxode2parse_unprotect();
     err_trans("nothing in output queue to write");
   }
-  if (!isString(c_file) || length(c_file) != 1){
+  if (!isString(c_file) || Rf_length(c_file) != 1){
+    _rxode2parse_unprotect();
     err_trans("c_file should only be 1 file");
   }
-  if (length(libname) != 2){
+  if (Rf_length(libname) != 2){
+    _rxode2parse_unprotect();
     err_trans("libname needs 2 elements");
   }
   fpIO = fopen(CHAR(STRING_ELT(c_file,0)), "wb");
   err_msg((intptr_t) fpIO, "error opening output c file\n", -2);
+
   if (badMd5){
     SET_STRING_ELT(VECTOR_ELT(mvLast, RxMv_md5), 0, mkChar(""));
   } else {
@@ -525,11 +623,25 @@ SEXP _rxode2_codegen(SEXP c_file, SEXP prefix, SEXP libname,
     SET_STRING_ELT(trans, 21, mkChar(buf.s)); // IndF
   }
   sPrint(&_mv, "%s", CHAR(STRING_ELT(PROTECT(_rxode2_rxQs(mvLast)), 0))); pro++;
-  UNPROTECT(pro);
   sFree(&buf);
   //SET_STRING_ELT(tran, 0, mkChar());
   sFree(&sbOut);
   sIniTo(&sbOut, (int)((sbPm.sN)*5.3));
+
+  SEXP stateOrd = PROTECT(VECTOR_ELT(mvLast, RxMv_stateOrd)); pro++;
+  int nOrd = Rf_length(stateOrd);
+  if (nOrd > 0) {
+    SEXP stateOrdNames = PROTECT(Rf_getAttrib(stateOrd, R_NamesSymbol)); pro++;
+    int *stateOrdInt = INTEGER(stateOrd);
+    sAppend(&sbOut, "// Define translation state order for %d states\n", Rf_length(stateOrd));
+    for (int i = 0; i < nOrd; i++){
+      sAppend(&sbOut, "#define __DDT%d__ %d // %s\n", stateOrdInt[i]-1, i,
+              CHAR(STRING_ELT(stateOrdNames, i)));
+    }
+    writeSb(&sbOut, fpIO);
+    sbOut.o = 0;
+  }
+  UNPROTECT(pro);
   // show_ode = 1 dydt
   // show_ode = 2 Jacobian
   // show_ode = 3 Ini statement
@@ -549,12 +661,15 @@ SEXP _rxode2_codegen(SEXP c_file, SEXP prefix, SEXP libname,
     }
     if (badCentral && badDepot){
       fclose(fpIO);
+      _rxode2parse_unprotect();
       err_trans("linCmt() and ode have 'central' and 'depot' compartments, rename ODE 'central'/'depot'");
     } else if (badCentral) {
       fclose(fpIO);
+      _rxode2parse_unprotect();
       err_trans("linCmt() and ode has a 'central' compartment, rename ODE 'central'");
     } else if (badDepot) {
       fclose(fpIO);
+      _rxode2parse_unprotect();
       err_trans("linCmt() and ode has a 'depot' compartment, rename ODE 'depot'");
     }
     (&sbOut)->s[0]='\0';
@@ -564,12 +679,14 @@ SEXP _rxode2_codegen(SEXP c_file, SEXP prefix, SEXP libname,
     } else if (tb.hasCentral == 1) {
       if (tb.hasDepot){
         fclose(fpIO);
+        _rxode2parse_unprotect();
         err_trans("linCmt() does not have 'depot' compartment without a 'ka'");
         return R_NilValue;
       }
       sAppend(&sbOut, "#define _CENTRAL_ %d\n", tb.statei);
     }
     writeSb(&sbOut, fpIO);
+    sbOut.o = 0;
   }
   gCode(1); // d/dt()
   gCode(2); // jac
