@@ -2,6 +2,13 @@
 ////////////////////////////////////////////////////////////////////////////////
 // rxode2 parsing function routines
 
+#define threadSafe 1
+#define threadSafeRepNumThread 2
+#define notThreadSafe 0
+
+
+SEXP rxode2_getUdf2(const char *fun, const int nargs);
+
 static inline int isAtFunctionArg(const char *name) {
   return !strcmp("(", name) ||
     !strcmp(")", name) ||
@@ -32,7 +39,7 @@ static inline int handleSimFunctions(nodeInfo ni, char *name, int *i, int nch,
 				     D_ParseNode *pn){
   if (nodeHas(simfun_statement) && *i == 0) {
     *i = nch; // done
-    if (tb.thread != 1) tb.thread = 2;
+    //if (tb.thread != threadSafe) tb.thread = threadSafeRepNumThread;
     sb.o=0;sbDt.o=0; sbt.o=0;
     D_ParseNode *xpn = d_get_child(pn, 0);
     char *v = (char*)rc_dup_str(xpn->start_loc.s, xpn->end);
@@ -75,12 +82,22 @@ typedef struct transFunctions {
   int isDiff;
   int isLinB;
   int isPnorm;
+
   int isTad;
+  int isTad0;
+
   int isTafd;
+  int isTafd0;
+
   int isTlast;
+  int isTlast0;
+
   int isDose;
   int isPodo;
+
   int isTfirst;
+  int isTfirst0;
+
   int isInd;
   nodeInfo ni;
   char *name;
@@ -110,12 +127,22 @@ static inline void transFunctionsIni(transFunctions *tf) {
   tf->isDiff=0;
   tf->isLinB=0;
   tf->isPnorm=0;
+
   tf->isTad=0;
+  tf->isTad0=0;
+
   tf->isTafd=0;
+  tf->isTafd0=0;
+
   tf->isTlast = 0;
+  tf->isTlast0 = 0;
+
   tf->isDose = 0;
   tf->isPodo = 0;
+
   tf->isTfirst = 0;
+  tf->isTfirst0 = 0;
+
   tf->isInd=0;
 }
 
@@ -165,18 +192,18 @@ static inline int handleFunctionLogit(transFunctions *tf) {
 static inline int handleFunctionSum(transFunctions *tf) {
   if (!strcmp("prod",tf->v)   || !strcmp("sum", tf->v) || !strcmp("sign",  tf->v) ||
       !strcmp("max", tf->v)   || !strcmp("min", tf->v) ||
-      !strcmp("rxord", tf->v)){
+      !strcmp("rxord", tf->v)) {
     int ii = d_get_number_of_children(d_get_child(tf->pn,3))+1;
-    if (!strcmp("prod", tf->v)){
+    if (!strcmp("prod", tf->v)) {
       sAppend(&sb, "_prod(_p, _input, _solveData->prodType, %d, (double) ", ii);
       sAppend(&sbDt, "_prod(_p, _input, _solveData->prodType, %d, (double) ", ii);
       if (maxSumProdN < ii){
         maxSumProdN = ii;
       }
-    } else if (!strcmp("sum", tf->v)){
+    } else if (!strcmp("sum", tf->v)) {
       sAppend(&sb, "_sum(_p, _pld, -__MAX_PROD__, _solveData->sumType, %d, (double) ", ii);
       sAppend(&sbDt, "_sum(_p, _pld, -__MAX_PROD__, _solveData->sumType, %d, (double) ", ii);
-      if (SumProdLD < ii){
+      if (SumProdLD < ii) {
         SumProdLD = ii;
       }
     } else if (!strcmp("rxord", tf->v)) {
@@ -220,6 +247,8 @@ static inline int handleFunctionsExceptLinCmt(transFunctions *tf) {
     handleFunctionIsInfinite(tf);
 }
 
+#define max2( a , b )  ( (a) > (b) ? (a) : (b) )
+
 static inline void handleLlFunctions(transFunctions *tf) {
   if (!strncmp("llikX", tf->v, 5)) {
     D_ParseNode *xpn = d_get_child(tf->pn,2);
@@ -230,12 +259,72 @@ static inline void handleLlFunctions(transFunctions *tf) {
   }
 }
 
-static inline void handleBadFunctions(transFunctions *tf) {
+#undef max2
+
+extern SEXP _rxode2parse_rxFunctionName;
+extern SEXP _rxode2parse_functionArgMin;
+extern SEXP _rxode2parse_functionArgMax;
+extern SEXP _rxode2parse_funName;
+extern SEXP _rxode2parse_funNameInt;
+extern SEXP _rxode2parse_functionThreadSafe;
+
+
+static inline int handleBadFunctions(transFunctions *tf) {
   // Split out to handle anticipated automatic conversion of R
   // functions to C
   int foundFun = 0;
-  for (int j = length(_goodFuns); j--;){
+  int curThread = 0;
+  for (int j = Rf_length(_goodFuns); j--;){
     if (!strcmp(CHAR(STRING_ELT(_goodFuns, j)),tf->v)){
+      int ii = d_get_number_of_children(d_get_child(tf->pn,3))+1;
+      int argMin=-1, argMax=-1;
+      for (int kk = Rf_length(_rxode2parse_rxFunctionName); kk--;) {
+        if (!strcmp(CHAR(STRING_ELT(_rxode2parse_rxFunctionName, kk)),tf->v)) {
+          argMin = INTEGER(_rxode2parse_functionArgMin)[kk];
+          argMax = INTEGER(_rxode2parse_functionArgMax)[kk];
+          curThread = INTEGER(_rxode2parse_functionThreadSafe)[kk];
+          if (curThread == 0) tb.thread = notThreadSafe;
+          if (argMin == NA_INTEGER || argMax == NA_INTEGER) {
+            argMin = argMax = -1;
+            break;
+          }
+          if (argMax < argMin) {
+            int tmp = argMax;
+            argMax = argMin;
+            argMin = tmp;
+          }
+          break;
+        }
+      }
+      if (argMin == -1) {
+        // Check for more known args
+        for (int kk = Rf_length(_rxode2parse_funName); kk--;) {
+          if (!strcmp(CHAR(STRING_ELT(_rxode2parse_funName, kk)),tf->v)) {
+            argMin = argMax = INTEGER(_rxode2parse_funNameInt)[kk];
+            if (argMin == NA_INTEGER) {
+              argMin=argMax=-1;
+              break;
+            }
+            break;
+          }
+        }
+      }
+      if (argMin != -1) {
+        if (argMin == argMax && argMin != ii) {
+          updateSyntaxCol();
+          sPrint(&_gbuf, _("'%s' takes %d arguments, supplied %d"),
+                 tf->v, argMin, ii);
+          /* Free(v2); */
+          trans_syntax_error_report_fn(_gbuf.s);
+          return 0;
+        } else if (argMin > ii || argMax < ii) {
+          sPrint(&_gbuf, _("'%s' takes %d-%d arguments, supplied %d"),
+                 tf->v, argMin, argMax, ii);
+          /* Free(v2); */
+          trans_syntax_error_report_fn(_gbuf.s);
+          return 0;
+        }
+      }
       // Save log-likelihood information
       handleLlFunctions(tf);
       foundFun = 1;
@@ -244,10 +333,36 @@ static inline void handleBadFunctions(transFunctions *tf) {
     }
   }
   if (foundFun == 0){
-    sPrint(&_gbuf, _("function '%s' is not supported in rxode2"), tf->v);
-    updateSyntaxCol();
-    trans_syntax_error_report_fn(_gbuf.s);
+    int ii = d_get_number_of_children(d_get_child(tf->pn,3))+1;
+    SEXP lst = PROTECT(rxode2_getUdf2(tf->v, ii));
+    int udf = INTEGER(VECTOR_ELT(lst, 0))[0];
+    const char *udfInfo = R_CHAR(STRING_ELT(VECTOR_ELT(lst, 1), 0));
+    UNPROTECT(1);
+    if (udf == NA_INTEGER) {
+      sPrint(&_gbuf, "%s", udfInfo);
+      updateSyntaxCol();
+      trans_syntax_error_report_fn(_gbuf.s);
+    } else {
+      if (udf != ii) {
+        sPrint(&_gbuf, _("user function '%s' takes %d arguments, supplied %d"),
+               tf->v, udf, ii);
+        updateSyntaxCol();
+        trans_syntax_error_report_fn(_gbuf.s);
+      } else {
+        if (maxUdf < ii){
+          maxUdf = ii;
+        }
+        sAppend(&sb, "_udf(\"%s\", __udf, %d, (double) ", tf->v, ii);
+        sAppend(&sbDt, "_udf(\"%s\", __udf, %d, (double) ", tf->v, ii);
+        sAppend(&sbt, "%s(", tf->v);
+        tb.thread = notThreadSafe;
+        tf->i[0] = 1;// Parse next arguments
+        tf->depth[0]=1;
+        return 1;
+      }
+    }
   }
+  return 0;
 }
 
 static inline int handleFunctions(nodeInfo ni, char *name, int *i, int *depth, int nch, D_ParseNode *xpn, D_ParseNode *pn) {
@@ -266,8 +381,8 @@ static inline int handleFunctions(nodeInfo ni, char *name, int *i, int *depth, i
       return 1;
     } else if (handleFunctionLinCmt(tf)){
       return 0;
-    } else {
-      handleBadFunctions(tf);
+    } else if (handleBadFunctions(tf)) {
+      return 1;
     }
   }
   return 0;
@@ -279,7 +394,7 @@ static inline int handlePrintf(nodeInfo ni, char *name, int i, D_ParseNode *xpn)
     if (i == 0){
       sb.o =0; sbDt.o =0;
       sbt.o=0;
-      tb.thread = 0;
+      tb.thread = notThreadSafe;
       aType(PPRN);
       aAppendN("Rprintf(", 8);
       sAppendN(&sbt,"printf(", 7);
