@@ -1,3 +1,5 @@
+#include "strncmpi.h"
+
 static inline int assertForbiddenVariables(const char *s) {
   if (!strcmp("printf", s)){
     updateSyntaxCol();
@@ -26,23 +28,25 @@ static inline int assertForbiddenVariables(const char *s) {
   }
   if (!strcmp("ifelse", s)){
     updateSyntaxCol();
+    _rxode2parse_unprotect();
     err_trans("'ifelse' cannot be a state in an rxode2 model");
     tb.ix=-2;
     return 0;
   }
   if (!strcmp("if", s)){
     updateSyntaxCol();
+    _rxode2parse_unprotect();
     err_trans("'if' cannot be a variable/state in an rxode2 model");
     tb.ix=-2;
     return 0;
   }
-  if (!strcmp("evid", s)){ // This is mangled by rxode2 so don't use it.
+  if (!rxstrcmpi("evid", s)){ // This is mangled by rxode2 so don't use it.
     updateSyntaxCol();
     trans_syntax_error_report_fn(_("'evid' cannot be a variable in an rxode2 model"));
     tb.ix=-2;
     return 0;
   }
-  if (!strcmp("ii", s)){ // This is internally driven and not in the
+  if (!rxstrcmpi("ii", s)){ // This is internally driven and not in the
   			 // covariate table so don't use it.
     updateSyntaxCol();
     trans_syntax_error_report_fn(_("'ii' cannot be a variable in an rxode2 model"));
@@ -52,8 +56,8 @@ static inline int assertForbiddenVariables(const char *s) {
   return 1;
 }
 static inline int isReservedVariable(const char *s) {
-  return !strcmp("amt", s) ||
-    !strcmp("time", s) ||
+  return !rxstrcmpi("amt", s) ||
+    !rxstrcmpi("time", s) ||
     !strcmp("rx__PTR__", s) ||
     !strcmp("tlast", s) ||
     // Ignore M_ constants
@@ -117,17 +121,34 @@ static inline int new_or_ith(const char *s) {
   if (skipReservedVariables(s) == 0) return 0;
   // Ignore THETA[] and ETA
   if (strstr("[", s) != NULL) {tb.ix=-2;return 0;}
+  if (!strcmp("rxlin___", s)) return 0;
 
   for (i=0; i<NV; i++) {
     if (!strcmp(tb.ss.line[i], s)) {
       tb.ix = i;
+      // if currently defining an interpolation and
+      // this variable has not been assigned an interpolation
+      // then assign it.
+      if (tb.interpC != 0) {
+        if (tb.interp[tb.ix] == 0) {
+          tb.interp[tb.ix] = tb.interpC;
+        } else {
+          // It this variable has already been assigned an
+          // interpolation error
+          sPrint(&_gbuf,"'%s' cannot have more than one interpolation method", s);
+          updateSyntaxCol();
+          trans_syntax_error_report_fn(_gbuf.s);
+        }
+      }
       return 0;
     }
   }
   if (NV+1 > tb.allocS){
     tb.allocS += MXSYM;
     tb.lh = R_Realloc(tb.lh, tb.allocS, int);
+    tb.interp = R_Realloc(tb.interp, tb.allocS, int);
     tb.lag = R_Realloc(tb.lag, tb.allocS, int);
+    tb.alag = R_Realloc(tb.alag, tb.allocS, int);
     tb.ini= R_Realloc(tb.ini, tb.allocS, int);
     tb.mtime=R_Realloc(tb.mtime, tb.allocS, int);
     tb.iniv=R_Realloc(tb.iniv, tb.allocS, double);
