@@ -5,28 +5,36 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdint.h>   /* dj: import intptr_t */
-#include "ode.h"
-#include "sbuf.h"
+//#include "ode.h"
+#include "../inst/include/rxode2.h"
+#include "../inst/include/rxode2parseSbuf.h"
 #include "getOption.h"
 #include "parseLinCmt.h"
 #include "tran.h"
 #include "genModelVars.h"
 #include "print_node.h"
 #include <errno.h>
-#include <dparser.h>
 #include <R.h>
 #include <Rinternals.h>
 #include <R_ext/Rdynload.h>
 #include <Rmath.h>
 #ifdef ENABLE_NLS
 #include <libintl.h>
-#define _(String) dgettext ("rxode2", String)
+#define _(String) dgettext ("rxode2parse", String)
 /* replace pkg as appropriate */
 #else
 #define _(String) (String)
 #endif
-#include "tran.g.d_parser.c"
-#include "../inst/include/rxode2.h"
+
+// change the name of the iniDparser pointer
+#define iniDparserPtr _rxode2_iniDparserPtr
+
+#include <dparserPtr.h>
+
+dparserPtrIni
+
+
+#include "tran.g.d_parser.h"
 
 #define MXSYM 50000
 #define MXDER 5000
@@ -57,13 +65,14 @@ int lastStrLoc=0;
 SEXP _goodFuns;
 vLines _dupStrs;
 
-int rx_syntax_error = 0, rx_suppress_syntax_info=0, rx_syntax_require_ode_first = 1;
+int rx_syntax_error = 0, rx_suppress_syntax_info=0;
 
-extern D_ParserTables parser_tables_rxode2;
+extern D_ParserTables parser_tables_rxode2parse;
 
 unsigned int found_jac = 0, nmtime=0;
-int rx_syntax_allow_ini = 1, 
-  maxSumProdN = 0, SumProdLD = 0, good_jac=1, extraCmt=0;
+int rx_syntax_allow_ini = 1,
+  maxSumProdN = 0, SumProdLD = 0, good_jac=1, extraCmt=0,
+  maxUdf=0;
 
 sbuf s_inits;
 
@@ -89,6 +98,7 @@ int firstErrD=0;
 
 vLines sbPm, sbPmDt, sbNrmL;
 sbuf sbNrm;
+sbuf sbExtra;
 vLines depotLines, centralLines;
 
 const char *model_prefix = NULL;
@@ -100,7 +110,7 @@ int foundF=0,foundLag=0, foundRate=0, foundDur=0, foundF0=0, needSort=0;
 sbuf sbOut;
 
 int lastSyntaxErrorLine=0;
-void updateSyntaxCol();
+void updateSyntaxCol(void);
 
 #include "parseVars.h"
 
@@ -110,7 +120,7 @@ int gBufLast = 0;
 D_Parser *curP=NULL;
 D_ParseNode *_pn = 0;
 
-void freeP(){
+void freeP(void){
   if (_pn){
     free_D_ParseTreeBelow(curP,_pn);
     free_D_ParseNode(curP,_pn);
@@ -128,18 +138,22 @@ int depotAttr=0, centralAttr=0;
 
 sbuf _gbuf, _mv;
 
-static inline int new_de(const char *s);
+static inline int new_de(const char *s, int fromWhere);
 static inline int handleRemainingAssignmentsCalcProps(nodeInfo ni, char *name, int i, D_ParseNode *pn, D_ParseNode *xpn, char *v);
 static inline int finalizeLineDdt(nodeInfo ni, char *name);
 static inline int finalizeLineParam(nodeInfo ni, char *name);
+static inline int finalizeLineInterp(nodeInfo ni, char *name);
 static inline int isCmtLhsStatement(nodeInfo ni, char *name, char *v);
 //static inline int add_deCmtProp(nodeInfo ni, char *name, char *v, int hasLhs, int fromWhere);
 static inline void add_de(nodeInfo ni, char *name, char *v, int hasLhs, int fromWhere);
 
+#include "parseAllowAssign.h"
 #include "parseFuns.h"
 #include "parseLogical.h"
 #include "parseIdentifier.h"
 #include "parseIndLin.h"
+#include "parseAssignStr.h"
+#include "parseLevels.h"
 #include "parseStatements.h"
 #include "parseDfdy.h"
 #include "parseCmtProperties.h"
@@ -159,6 +173,7 @@ static inline int parseNodePossiblySkipRecursion(nodeInfo ni, char *name, D_Pars
       handleSimFunctions(ni, name, i, nch, pn) ||
       handleStringEqualityStatements(ni, name, *i, xpn) ||
       handleDvidStatement(ni, name, xpn, pn) ||
+      handleStartInterpStatement(ni, name, i, xpn, pn) ||
       handleFunctions(ni, name, i, depth, nch, xpn, pn) ||
       handleTheta(ni, name, xpn) ||
       handleEta(ni, name, xpn)) return 1;
@@ -174,9 +189,12 @@ static inline int parseNodeAfterRecursion(nodeInfo ni, char *name, D_ParseNode *
       handleLogicalExpr(ni, name, *i, pn, xpn, isWhile) ||
       handleCmtProperty(ni, name, *i, xpn) ||
       handleDdtAssign(ni, name, *i, pn, xpn) ||
-      handleDdtRhs(ni, name, xpn)) return 1;
+      handleDdtRhs(ni, name, xpn) ||
+      handleStrAssign(ni, name, *i, pn, xpn) ||
+      handleLevelsStr(ni, name, *i, pn, xpn) ||
+      handleLevelsStr1(ni, name, *i, pn, xpn)) return 1;
   if (*i==0 && nodeHas(power_expression)) {
-    aAppendN("),", 2);
+    aAppendN(",", 1);
     sAppendN(&sbt, "^", 1);
   }
   handleRemainingAssignments(ni, name, *i, pn, xpn);
@@ -195,20 +213,17 @@ void wprint_parsetree(D_ParserTables pt, D_ParseNode *pn, int depth, print_node_
   handleFunctionArguments(name, depth);
   // print/change identifier/operator and change operator information (if needed)
   handleOperatorsOrPrintingIdentifiers(depth, fn, client_data, ni, name, value);
-
+  if (handleLevelStr(ni, name, value)) return;
   if (nch != 0) {
     int isWhile=0;
     if (nodeHas(power_expression)) {
-      aAppendN("Rx_pow(_as_dbleps(", 18);
+      aAppendN("Rx_pow(", 7);
     }
     for (i = 0; i < nch; i++) {
       D_ParseNode *xpn = d_get_child(pn, i);
-
       if (parseNodePossiblySkipRecursion(ni, name, pn, xpn, &i, nch, &depth)) continue;
-
       // Recursively parse tree
       wprint_parsetree(pt, xpn, depth, fn, client_data);
-
       parseNodeAfterRecursion(ni, name, pn, xpn, &i, nch, &depth, &safe_zero,
 			      &ii, &found, &isWhile);
     }
@@ -220,13 +235,14 @@ void err_msg(int chk, const char *msg, int code)
 {
   if(!chk) {
     parseFree(0);
+    _rxode2parse_unprotect();
     Rf_errorcall(R_NilValue, "%s",msg);
   }
 }
 
 sbuf _bufw, _bufw2;
 
-void parseFreeLast() {
+void parseFreeLast(void) {
   if (gBufFree) R_Free(gBuf);
   sFree(&sbOut);
   freeP();
@@ -234,34 +250,51 @@ void parseFreeLast() {
   sFree(&_bufw2);
 }
 
-void parseFree(int last){
+sbuf sbErr1;
+sbuf sbErr2;
+
+void parseFree(int last) {
   sFree(&sb);
   sFree(&sbDt);
   sFree(&sbt);
   sFree(&sbNrm);
+  sFree(&sbExtra);
   sFree(&s_inits);
   sFree(&_bufw);
   sFree(&_bufw2);
   sFree(&firstErr);
   sFree(&_gbuf);
   sFree(&_mv);
+  sFree(&sbErr1);
+  sFree(&sbErr2);
   lineFree(&sbPm);
   lineFree(&sbPmDt);
   lineFree(&sbNrmL);
   lineFree(&(tb.ss));
   lineFree(&(tb.de));
+  lineFree(&(tb.str));
+  lineFree(&(tb.strVal));
   lineFree(&depotLines);
   lineFree(&centralLines);
   lineFree(&_dupStrs);
   linCmtGenFree(&_linCmtGenStruct);
   R_Free(tb.lh);
+  R_Free(tb.interp);
   R_Free(tb.lag);
+  R_Free(tb.alag);
   R_Free(tb.ini);
   R_Free(tb.mtime);
   R_Free(tb.iniv);
   R_Free(tb.ini0);
   R_Free(tb.di);
+  R_Free(tb.didx);
+  R_Free(tb.dprop);
+  R_Free(tb.si);
+  R_Free(tb.sin);
+  R_Free(tb.strValI);
+  R_Free(tb.strValII);
   R_Free(tb.idi);
+  R_Free(tb.isi);
   R_Free(tb.idu);
   R_Free(tb.dvid);
   R_Free(tb.df);
@@ -273,8 +306,24 @@ void parseFree(int last){
   }
 }
 
+SEXP _rxode2_parseFreeSexp(SEXP last) {
+  parseFree(INTEGER(last)[0]);
+  return R_NilValue;
+}
 
-void reset() {
+char *alagLinCmtLine = NULL;
+char *fLinCmtLine = NULL;
+char *durLinCmtLine = NULL;
+char *rateLinCmtLine = NULL;
+
+char *alag1LinCmtLine = NULL;
+char *f1LinCmtLine = NULL;
+char *rate1LinCmtLine = NULL;
+char *dur1LinCmtLine = NULL;
+
+SEXP _rxode2_resetUdf(void);
+
+void reset(void) {
   // Reset sb/sbt string buffers
   parseFree(0);
   sIniTo(&_bufw, 1024);
@@ -283,7 +332,10 @@ void reset() {
   sIniTo(&sbDt, MXDER);
   sIniTo(&sbt, SBUF_MXBUF);
   sIniTo(&sbNrm, SBUF_MXBUF);
+  sIniTo(&sbExtra,SBUF_MXBUF);
   sIniTo(&_gbuf, 1024);
+  sIniTo(&sbErr1, SBUF_MXBUF);
+  sIniTo(&sbErr2, SBUF_MXBUF);
   sIni(&_mv);
   sClear(&_mv);
   sIniTo(&firstErr, SBUF_MXBUF);
@@ -300,16 +352,29 @@ void reset() {
 
   lineIni(&(tb.ss));
   lineIni(&(tb.de));
+  lineIni(&(tb.str));
 
   tb.lh		= R_Calloc(MXSYM, int);
+  tb.interp	= R_Calloc(MXSYM, int);
+  tb.interpC= 0;
   tb.ini	= R_Calloc(MXSYM, int);
   tb.mtime	= R_Calloc(MXSYM, int);
   tb.iniv	= R_Calloc(MXSYM, double);
   tb.ini0	= R_Calloc(MXSYM, int);
   tb.di		= R_Calloc(MXDER, int);
+  tb.didx   = R_Calloc(MXDER, int);
+  tb.dprop  = R_Calloc(MXDER, int);
+  tb.didxn  = 1;
+  tb.si     = R_Calloc(MXDER, int);
+  tb.sin    = R_Calloc(MXDER, int);
   tb.idi	= R_Calloc(MXDER, int);
+  tb.strValI= R_Calloc(MXDER, int);
+  tb.strValII= R_Calloc(MXDER, int);
+  tb.isi    = R_Calloc(MXDER, int);
   tb.idu	= R_Calloc(MXDER, int);
   tb.lag	= R_Calloc(MXSYM, int);
+  tb.alag   = R_Calloc(MXSYM, int);
+  tb.alagn  = 0;
   tb.dvid	= R_Calloc(MXDER, int);
   tb.thread     = 1; // Thread safe flag
   tb.dvidn      = 0;
@@ -347,6 +412,8 @@ void reset() {
   tb.hasKa      = 0;
   tb.allocS	= MXSYM;
   tb.allocD	= MXDER;
+  tb.allocS = MXDER;
+  tb.allocSV = MXDER;
   tb.matn	= 0;
   tb.matnf	= 0;
   tb.ncmt	= 0;
@@ -356,6 +423,7 @@ void reset() {
   tb.centralN	= -1;
   tb.linExtra   = false;
   tb.nwhile     = 0;
+  tb.lvlStr     = 0;
   tb.nInd       = 0;
   tb.simflg     = 0;
   tb.nLlik      = 0;
@@ -388,11 +456,23 @@ void reset() {
   nmtime=0;
   syntaxErrorExtra=0;
   extraCmt=0;
+
+  // reset extra lines from linCmt()
+  alagLinCmtLine = NULL;
+  fLinCmtLine = NULL;
+  durLinCmtLine = NULL;
+  rateLinCmtLine = NULL;
+
+  alag1LinCmtLine = NULL;
+  f1LinCmtLine = NULL;
+  rate1LinCmtLine = NULL;
+  dur1LinCmtLine = NULL;
+  _rxode2_resetUdf();
 }
 
 static void rxSyntaxError(struct D_Parser *ap);
 
-static inline void assertCorrectDfDy() {
+static inline void assertCorrectDfDy(void) {
   char *buf1, *buf2, bufe[2048];
   int i, j, found, islhs;
   for (i=0; i<tb.ndfdy; i++) {                     /* name state vars */
@@ -445,7 +525,7 @@ static inline void assertCorrectDfDy() {
 
 void trans_internal(const char* parse_file, int isStr){
   freeP();
-  curP = new_D_Parser(&parser_tables_rxode2, sizeof(D_ParseNode_User));
+  curP = new_D_Parser(&parser_tables_rxode2parse, sizeof(D_ParseNode_User));
   curP->save_parse_tree = 1;
   curP->error_recovery = 1;
   curP->initial_scope = NULL;
@@ -463,7 +543,9 @@ void trans_internal(const char* parse_file, int isStr){
     err_msg((intptr_t) gBuf, "error: empty buf for FILE_to_parse\n", -2);
   }
   sFree(&sbNrm);
+  sFree(&sbExtra);
   sIniTo(&sbNrm, SBUF_MXBUF);
+  sIniTo(&sbExtra, SBUF_MXBUF);
   lineIni(&sbPm);
   lineIni(&sbPmDt);
   lineIni(&sbNrmL);
@@ -475,15 +557,17 @@ void trans_internal(const char* parse_file, int isStr){
   if (!_pn || curP->syntax_errors) {
     rx_syntax_error = 1;
   } else {
-    wprint_parsetree(parser_tables_rxode2, _pn, 0, wprint_node, NULL);
+    wprint_parsetree(parser_tables_rxode2parse, _pn, 0, wprint_node, NULL);
     // Determine Jacobian vs df/dvar
     assertCorrectDfDy();
   }
 }
 
+
+extern int _rxode2parse_protected;
 static inline int setupTrans(SEXP parse_file, SEXP prefix, SEXP model_md5, SEXP parseStr,
                              SEXP isEscIn, SEXP inME, SEXP goodFuns, SEXP fullPrintIn) {
-  _goodFuns = goodFuns;
+  _goodFuns = PROTECT(goodFuns); _rxode2parse_protected++;
   // Make sure buffers are initialized.
   isEsc=INTEGER(isEscIn)[0];
   fullPrint=INTEGER(fullPrintIn)[0];
@@ -492,25 +576,26 @@ static inline int setupTrans(SEXP parse_file, SEXP prefix, SEXP model_md5, SEXP
   reset();
   rx_suppress_syntax_info = R_get_option("rxode2.suppress.syntax.info",0);
   rx_syntax_allow_ini  = R_get_option("rxode2.syntax.allow.ini",1);
-  rx_syntax_require_ode_first = R_get_option("rxode2.syntax.require.ode.first",1);
   set_d_use_r_headers(0);
   set_d_rdebug_grammar_level(0);
   set_d_verbose_level(0);
 
-  if (isString(prefix) && length(prefix) == 1){
+  if (isString(prefix) && Rf_length(prefix) == 1){
     model_prefix = CHAR(STRING_ELT(prefix,0));
   } else {
+    _rxode2parse_unprotect();
     err_trans("model prefix must be specified");
   }
 
-  if (isString(inME) && length(inME) == 1){
+  if (isString(inME) && Rf_length(inME) == 1){
     me_code = CHAR(STRING_ELT(inME,0));
   } else {
     freeP();
+    _rxode2parse_unprotect();
     err_trans("extra ME code must be specified");
   }
 
-  if (isString(model_md5) && length(model_md5) == 1){
+  if (isString(model_md5) && Rf_length(model_md5) == 1){
     md5 = CHAR(STRING_ELT(model_md5,0));
     badMd5 = 0;
     if (strlen(md5)!= 32){
@@ -522,7 +607,7 @@ static inline int setupTrans(SEXP parse_file, SEXP prefix, SEXP model_md5, SEXP
   return isStr;
 }
 
-static inline void finalizeSyntaxError() {
+static inline void finalizeSyntaxError(void) {
   if (rx_syntax_error){
     if(!rx_suppress_syntax_info){
       if (gBuf[gBufLast] != '\0'){
@@ -545,41 +630,49 @@ static inline void finalizeSyntaxError() {
     }
     if (firstErrD == 1) {
       firstErrD=0;
+      _rxode2parse_unprotect();
       err_trans(firstErr.s);
     } else {
+      _rxode2parse_unprotect();
       err_trans("syntax errors (see above)");
     }
   }
 }
 
+void _rxode2parse_assignTranslation(SEXP df);
+SEXP getRxode2ParseDf(void);
+
 SEXP _rxode2_trans(SEXP parse_file, SEXP prefix, SEXP model_md5, SEXP parseStr,
                    SEXP isEscIn, SEXP inME, SEXP goodFuns, SEXP fullPrintIn){
   const char *in = NULL;
+  _rxode2parse_assignTranslation(getRxode2ParseDf());
   int isStr = setupTrans(parse_file, prefix, model_md5, parseStr, isEscIn, inME, goodFuns, fullPrintIn);
   in = CHAR(STRING_ELT(parse_file,0));
   trans_internal(in, isStr);
   SEXP lst = PROTECT(generateModelVars());
   finalizeSyntaxError();
   UNPROTECT(1);
+  _rxode2parse_unprotect();
   return lst;
 }
 
 SEXP _rxode2_parseModel(SEXP type){
   if (!sbPm.o){
+    _rxode2parse_unprotect();
     err_trans("model no longer loaded in memory");
   }
   int iT = INTEGER(type)[0];
   SEXP pm;
   switch (iT){
   case 1:
-    pm = PROTECT(allocVector(STRSXP, sbPmDt.n));
+    pm = PROTECT(Rf_allocVector(STRSXP, sbPmDt.n));
     for (int i = 0; i < sbPmDt.n; i++){
       SET_STRING_ELT(pm, i, mkChar(sbPmDt.line[i]));
     }
     break;
-    
+
   default:
-    pm = PROTECT(allocVector(STRSXP, sbPm.n));
+    pm = PROTECT(Rf_allocVector(STRSXP, sbPm.n));
     for (int i = 0; i < sbPm.n; i++){
       SET_STRING_ELT(pm, i, mkChar(sbPm.line[i]));
     }
@@ -589,8 +682,8 @@ SEXP _rxode2_parseModel(SEXP type){
   return pm;
 }
 
-SEXP _rxode2_codeLoaded(){
-  SEXP pm = PROTECT(allocVector(INTSXP, 1));
+SEXP _rxode2_codeLoaded(void){
+  SEXP pm = PROTECT(Rf_allocVector(INTSXP, 1));
   if (!sbPm.o || !sbNrm.o){
     INTEGER(pm)[0]=0;
   } else {
@@ -600,8 +693,8 @@ SEXP _rxode2_codeLoaded(){
   return pm;
 }
 
-SEXP _rxode2_isLinCmt(){
-  SEXP ret = PROTECT(allocVector(INTSXP, 1));
+SEXP _rxode2_isLinCmt(void) {
+  SEXP ret = PROTECT(Rf_allocVector(INTSXP, 1));
   INTEGER(ret)[0]=tb.linCmt;
   UNPROTECT(1);
   return ret;
@@ -621,15 +714,18 @@ char * rc_dup_str(const char *s, const char *e) {
   return _dupStrs.line[_dupStrs.n-1];
 }
 
-void transIniNull() {
+void transIniNull(void) {
   sNull(&(s_inits));
   lineNull(&(tb.ss));
   lineNull(&(tb.de));
+  lineNull(&(tb.str));
+  lineNull(&(tb.strVal));
   sNull(&(sb));
   sNull(&(sbDt));
   sNull(&(sbt));
   sNull(&(firstErr));
   sNull(&(sbNrm));
+  sNull(&(sbExtra));
   sNull(&(sbOut));
   lineNull(&(sbPm));
   lineNull(&(sbPmDt));
