@@ -3,19 +3,23 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-extern int rx_syntax_require_ode_first, needSort;
-void updateSyntaxCol();
+extern int needSort;
+void updateSyntaxCol(void);
 void trans_syntax_error_report_fn(char *err);
 void parseFree(int last);
 void RSprintf(const char *format, ...);
 
-SEXP _rxode2_trans(SEXP parse_file, SEXP prefix, SEXP model_md5, SEXP parseStr,
-                   SEXP isEscIn, SEXP inME, SEXP goodFuns, SEXP fullPrintIn);
+SEXP _rxode2parse_trans(SEXP parse_file, SEXP prefix, SEXP model_md5, SEXP parseStr,
+                        SEXP isEscIn, SEXP inME, SEXP goodFuns, SEXP fullPrintIn);
 
 typedef struct symtab {
   vLines ss; // Symbol string or symbol lines
   /* char ss[64*MXSYM]; */                     /* symbol string: all vars*/
   vLines de;             /* symbol string: all Des*/
+  vLines str; /* symbol string: all assigned string variables*/
+  vLines strVal; /* symbol string for rxode2 assigned strings */
+  int *strValI; /* which variable is assigned a string */
+  int *strValII; /* The number that the string is assigned (what C sees) */
   int *lh;        /*
 lhs symbols?
 =0 not LHS
@@ -27,14 +31,29 @@ lhs symbols?
 =29
 =70 LHS + param
 */
+  int *interp;  // Interpolation flag:
+  // 0, = default via any declaration including param()
+  // 1 = linear via linear() or lin()
+  // 2 = constant/locf  via constant() or locf()
+  // 3 = nocb via nocb()
+  // 4 = midpoint via midpoint()
+  int interpC;// = 0; // current interpolation method
   int *ini;        /* initial variable assignment =2 if there are two assignments */
   int *mtime;
   double *iniv;        /* Initial values */
   int *ini0;        /* state initial variable assignment =2 if there are two assignments */
   int *di;        /* ith of state vars */
+  int *didx;
+  int *dprop;  /* property of state vars */
+  int didxn; /* nth of declared states */
+  int *si;      /* ith of string vars */
+  int *sin;      /* n values in each string var */
   int *idi;       /* should ith state variable be ignored 0/1 */
+  int *isi;      /* should ith string variable be ignored 0/1 */
   int *idu;       /* Has the ith state been used in a derivative expression? */
   int *lag;  // Lag number (if present)
+  int *alag; // absorption lag compartments seen
+  int alagn;
   int *dvid;
   int dvidn;
   int ix;                       /* ith of curr symbol */
@@ -71,6 +90,7 @@ lhs symbols?
   int hasCentralCmt;
   int hasKa;
   int allocS;
+  int allocSV;
   int allocD;
   int matn;
   int matnf;
@@ -88,6 +108,7 @@ lhs symbols?
   int thread;
   int lastDdt;
   int nLlik;
+  int lvlStr;
 } symtab;
 
 extern symtab tb;
@@ -96,8 +117,6 @@ extern vLines depotLines;
 extern vLines centralLines;
 extern vLines sbPm, sbPmDt, sbNrmL;
 
-
-
 #define FBIO 1
 #define ALAG 2
 #define RATE 3
@@ -119,10 +138,12 @@ extern vLines sbPm, sbPmDt, sbNrmL;
 #define TMAT0 19
 #define TMATF 20
 #define TLIN 21
+#define TNONE 22
 
 // new de type
 #define fromDDT 2
 #define fromCMTprop 1
+#define fromCMT 3
 
 #define NEEDSEMI _("lines need to end with ';'\n     to match R's handling of line endings set 'options(rxode2.syntax.require.semicolon = FALSE)'")
 #define NOSTATE _("defined 'df(%s)/dy(%s)', but '%s' is not a state")
@@ -134,6 +155,9 @@ extern vLines sbPm, sbPmDt, sbNrmL;
 typedef struct nodeInfo {
   int alag;
   int assignment;
+  int assign_str;
+  int levels_str;
+  int levels_str1;
   int constant;
   int der_rhs;
   int derivative;
@@ -176,6 +200,7 @@ typedef struct nodeInfo {
   int theta;
   int cmt_statement;
   int param_statement;
+  int interp_statement;
   int dvid_statementI;
   int ifelse;
   int ifelse_statement;
@@ -184,12 +209,17 @@ typedef struct nodeInfo {
   int equality_str1;
   int equality_str2;
   int simfun_statement;
+  int relational_op;
+  int string;
 } nodeInfo;
 
 static inline void niReset(nodeInfo *ni){
   ni->mtime = -1;
   ni->alag = -1;
   ni->assignment = -1;
+  ni->assign_str = -1;
+  ni->levels_str = -1;
+  ni->levels_str1 = -1;
   ni->constant = -1;
   ni->der_rhs = -1;
   ni->derivative = -1;
@@ -232,6 +262,7 @@ static inline void niReset(nodeInfo *ni){
   ni->theta0_noout = -1;
   ni->cmt_statement = -1;
   ni->param_statement = -1;
+  ni->interp_statement = -1;
   ni->dvid_statementI = -1;
   ni->ifelse = -1;
   ni->ifelse_statement=-1;
@@ -240,6 +271,8 @@ static inline void niReset(nodeInfo *ni){
   ni->equality_str1 = -1;
   ni->equality_str2 = -1;
   ni->simfun_statement = -1;
+  ni->relational_op = -1;
+  ni->string = -1;
 }
 
 #define STRINGIFY(...) STRINGIFY_AUX(__VA_ARGS__)
@@ -252,8 +285,10 @@ extern sbuf sbOut;
 
 #define notLHS 0
 #define isLHS 1
+#define isLHSstr 100
 #define isState 9
 #define isSuppressedLHS 10
+#define isSuppressedLHSstr 110
 #define isSuppressedParam 11
 #define isLhsStateExtra 19
 #define isLHSparam 70
@@ -261,15 +296,14 @@ extern sbuf sbOut;
 extern sbuf _gbuf, _mv;
 
 #define SBPTR sb.s+sb.o
-#define SBTPTR sbt.s+sbt.o
 #define NV tb.ss.n
 
 extern char *gBuf;
 extern int gBufFree;
 extern int gBufLast;
 
-extern int maxSumProdN, SumProdLD, foundF0, foundF, foundLag, foundRate, foundDur, 
-  good_jac, extraCmt, badMd5;
+extern int maxSumProdN, SumProdLD, foundF0, foundF, foundLag, foundRate, foundDur,
+  good_jac, extraCmt, badMd5, maxUdf;
 extern unsigned int found_jac, nmtime;
 
 extern sbuf sbNrm;
@@ -283,7 +317,7 @@ extern const char *md5;
 extern const char *model_prefix;
 extern const char *me_code;
 
-void reset();
+void reset(void);
 char * rc_dup_str(const char *s, const char *e);
 
 void trans_syntax_error_report_fn(char *err);
@@ -329,8 +363,16 @@ static inline int allSpaces(char *v2) {
 
 void parseFree(int last);
 
-#define err_trans(chr) Rf_errorcall(R_NilValue, _(chr));
+#define err_trans(chr) Rf_errorcall(R_NilValue, "%s", _(chr));
+
+void _rxode2parse_unprotect(void);
 
 char *getLine (char *src, int line, int *lloc);
 
+#define prop0    1
+#define propF    2
+#define propAlag 4
+#define propRate 8
+#define propDur  16
+
 #endif // __TRAN_H__
